import { SecurityManager } from "zen-antigen";
import { changeLocale, getServerLocale } from "zen-lingual";
import { ServiceLocator, noop, api } from "zen-cerebellum";

import {
  USER_AUTH,
  USER_LOG_OUT,
  USER_SESSION_RESTORE,
  HISTORY_USER_DISCONNECT
} from "../constants";

import { encodePassword } from "./password";

//WAA : logs user disconnection
const historizeUserDisconnection = () =>
  api.post(HISTORY_USER_DISCONNECT, "/history/logUserDisconnection");

//RAA : proceed to user login
export function loginUser(
  instance,
  login,
  password,
  remember = false,
  resetPassword = ""
) {
  return dispatch => {
    SecurityManager.changeSync(remember);

    return dispatch(
      api.post(USER_AUTH, "/auth", {
        Login: login,
        Instance: instance,
        Password: encodePassword(password),
        NewPassword: encodePassword(resetPassword),
        remember
      })
    ).then(jwt => {
      //init session from token
      dispatch(restoreSession(jwt));
    });
  };
}

//RAA : proceed to manual user exit
export const logoutUser = () => dispatch => {
  return dispatch(historizeUserDisconnection()).then(() => {
    dispatch(disconnectUser());
  });
};

/** FSA : force current user to be logged out. (to be used by non-public actions)
  @param message : string to display when user logs out
*/
export const disconnectUser = (message = "") => {
  SecurityManager.logout();
  return {
    type: USER_LOG_OUT,
    payload: {
      message
    }
  };
};

/** FSA : restore user from token if provided, otherwise from local storage
  @param token (string) : JWT token
*/
const restoreUser = token => {
  const policy = SecurityManager.init();
  const validToken = policy.restore(token);

  //then sync with session tree
  return {
    type: USER_SESSION_RESTORE,
    payload: {
      auth: validToken && policy.authorized(),
      user: policy.login,
      instance: policy.instance,
      remember: policy.isPersistent(),
      token: policy.jwt
    }
  };
};

/** RAA : restore session (user + settings) from token if provided, otherwise from local storage
  @param token (string) : JWT token
*/
export const restoreSession = token => {
  //getState is called several times betweeen dispatches to ensure fresh state information.
  return (dispatch, getState) => {
    //restore user first
    dispatch(restoreUser(token));

    //end here if token has not been restored yet because will fail otherwise (restoreSession being call early during initialization flow)
    const state = getState();
    if (state && !state.session.token) return noop();

    //get callback to be fired when user session is restored.
    const sessionCallback = ServiceLocator.get("sessionCallback");
    if (!sessionCallback)
      throw new Error(
        `A session callback declaration is missing in ServiceLocator.`
      );

    //dispatch the callback (action creator expected)
    return dispatch(sessionCallback()).then(() => {
      //retrieve back-office locale selector
      const extractLang = ServiceLocator.get("extractLang") || getServerLocale;
      //change current locale in order to be sync with server one
      dispatch(changeLocale(extractLang(getState())));
    });
  };
};
