import { api } from "zen-cerebellum";
import {
  TRANSFER_FILE,
  SNAPSHOT_DOCUMENT,
  REFRESH_ENTITIES,
} from "../constants";

import { computeURL } from "../api/helper";
import { getEndpoint, Kind } from "../utils/core";
import { withMultiSelection } from "./selection";
import { browseTo } from "./router";

/** Compute back-office WebAPI URL for document 'actions'
 * @param {object} state - store state tree
 * @param {object} document - document entity model
 * @param {string} mode - (optional) final option mode for URL computation
 * @returns {string} computed URL
 */
function getDocumentURL(state, document, mode = null) {
  return computeURL(state, getEndpoint(document, mode));
}

/** RAA : compute the URL (preview PDF) of the document
 * @param {object} doc - document entity
 * @param {object} options - display options
 *    @param {bool} [native] - true if native viewer shall be used instead of ZenPDF viewer.
 * @return {string} URL of document PDF
 */
export function getPreviewURL(doc, { native = false } = {}) {
  return (_, getState) => {
    const url = getDocumentURL(getState(), doc, "preview");

    //force inline content disposition for native viewer
    if (native) return `${url}&disposition=inline`;

    //build settings expected by PDF viewer (JSON string encoded in base 64)
    const viewerSettings = btoa(
      JSON.stringify({
        url,
        name: encodeURIComponent(doc.Lab || doc.Id),
      })
    );

    //open external viewer with settings
    return `/pdf/viewer.html?${viewerSettings}`;
  };
}

/** RAA : compute the URL of the public static resource
 * @param {string} resxId - Resource unique identifier
 * @param {object} options - custom options
 *    @param {string} [disposition] - as resource content disposition (default = inline)
 * @return {string} URL of static 'document' resource
 */
export function getStaticURL(resxId, { disposition = "inline" } = {}) {
  return (_, getState) => {
    if (resxId) {
      //retrieve static URL like a fake document
      const doc = { Kind: Kind.Document, Id: resxId };
      const url = getDocumentURL(getState(), doc, "static");
      return `${url}${disposition ? `&disposition=${disposition}` : ""}`;
    }
  };
}

/** RAA : compute the URL (image thumbnail) of the document
 * @param {object} doc - document entity
 * @return {string} URL of document thumbnail
 */
export function getThumbnailURL(doc) {
  return (_, getState) => {
    if (doc) {
      return getDocumentURL(getState(), doc, "thumbnail");
    }
  };
}

/**
 * RAA - Force the download of the document
 * @param {object} doc - document entity
 * @param {bool} raw - true for raw text
 */
export function downloadDocument(doc, raw) {
  return downloadFrom(getEndpoint(doc, raw ? "rawText" : "stream"));
}

/**
 * Trigger a download from given API endpoint
 */
export function downloadFrom(endpoint) {
  return (_, getState) => {
    const url = computeURL(getState(), endpoint);
    browseTo(url);
  };
}

/** RAA - Display document in browser or PDF viewer
 * @param {object} doc - document entity
 * @param {string} target - target attribute for the new tab (_blank, _self)
 * @param {object} options - display options
 */
export function displayDocument(doc, target = "_self", options) {
  return (dispatch) => {
    if (doc) {
      const url = dispatch(getPreviewURL(doc, options));
      browseTo(url, target);
    }
  };
}

//WAA : load document as snapshot
export function loadSnapshot(doc) {
  return api.get(SNAPSHOT_DOCUMENT, getEndpoint(doc, "snapshot"));
}

//RAA : download docs as zip
export const downloadZip = withMultiSelection(
  (key, action, state) => computeURL(state, `/selection/${key}/${action}`),
  { open: "archive" }
);

/**
 * Factory to create custom upload function.
 * @param {func} endpointBuilder - compute upload endpoint
 * @param {string} [type] - Overriden REDUX API type
 * @return {func} dispatchable upload
 */
function uploader(endpointBuilder, type = TRANSFER_FILE) {
  /**
   * RAA - Launch an file upload to a container entity
   * @param {FileAPI} file - file to upload  https://developer.mozilla.org/fr/docs/Web/API/File
   * @param {object} target - target entity where to index new document
   * @param {func} [onProgress] - callback for progression notification
   * @param {object} [params] - options parameters to send with upload
   */
  return function upload(file, target, onProgress, params = {}) {
    return (dispatch) => {
      if (!file) return;

      //inject file property into form data object
      const formData = new FormData();
      formData.append("file", file);

      //inject all parameters in form
      Object.keys(params).forEach((p) => formData.append(p, params[p]));

      return dispatch(
        api.post(
          type,
          endpointBuilder(target),
          formData,
          { entity: target },
          { onProgress }
        )
      );
    };
  };
}

//WAA : upload new document into target
export const uploadFile = uploader(() => `/upload`);

//WAA : replace document file
export const replaceFile = uploader((doc) => getEndpoint(doc, "replace"));

/**
 * WAA - Fetch information for a single entity
 * @param {entity} entity
 */
export const refreshEntity = (e) =>
  //refresh is just fetch again but with keeping reference of old' entity
  api.get(REFRESH_ENTITIES, getEndpoint(e), null, { entity: e });
