import { createBrowserHistory } from "history";

//singleton browser history
export const history = createBrowserHistory();

/** RAA : Navigate to a specific page location within current SPA
  @param {string} location - where to navigate to (can be an object see HTML History API)
  @param {bool} replace - True to replace current URL history, False to append.
*/
export function goto(location, replace = false) {
  return () => {
    if (replace) {
      history.replace(location);
    } else {
      history.push(location);
    }
  };
}

/**
 * Parse the URL query string to retrieve corresponding variable.
 * @param {string} location - URL location part (search/hash fragment)
 * @param {string} variable - Variable to check for.
 */
export function getQueryValue(locationPart, variable) {
  const query = locationPart.substring(1);
  const vars = query.split("&");
  for (let i = 0; i < vars.length; i++) {
    const pair = vars[i].split("=");
    if (decodeURIComponent(pair[0]) === variable) {
      return decodeURIComponent(pair[1]);
    }
  }
}

/**
 * Recursive helper to encode complex element into query string.
 */
function buildParam(prefix, item, add) {
  if (Array.isArray(item)) {
    item.forEach((element) => {
      buildParam(`${prefix}[]`, element, add);
    });
  } else if (item instanceof Object) {
    Object.keys(item).forEach((key) => {
      buildParam(`${prefix}[${key}]`, item[key], add);
    });
  } else {
    add(prefix, item);
  }
}

/**
 * Encode a payload into compatible query string.
 * @param {any} payload
 * @return {string} Query string.
 */
export function encodeToQueryString(payload) {
  const params = [];

  //queue new param to encoding list
  function addParam(prefix, value) {
    if (value !== undefined) {
      params[params.length] = `${encodeURIComponent(
        prefix
      )}=${encodeURIComponent(value === null ? "" : value)}`;
    }
  }

  Object.keys(payload).forEach((key) => {
    buildParam(key, payload[key], addParam);
  });

  return `?${params.join("&")}`;
}

/**
 * Manage browser window opening.
 * @param {string} url
 * @param {string} target (_self, _blank, name)
 */
export function browseTo(url, target = "_self", features = "") {
  return window.open(url, target, features, true);
}
