import { nls } from "zen-lingual";

import { INIT, DONE, FAIL } from "../constants";
import { selectToken } from "../utils/core";
import { logInternalError } from "../actions/notifications";
import { disconnectUser } from "../actions/auth";
import { goto } from "../actions/router";
import { computeURL, httpVerbs, makeInternalError } from "./helper";
import fetch from "./fetch";

/**
 * Redux middleware that interprets 'API' action to perform web API call and handle response (including Result<T>).
 * Provides an extension point between dispatching an action, and the moment it reaches the reducer.
 *
 * @param {function} store - redux state tree storage
 * @param {function} next - next middleware's dispatch function
 * @param {object} action - current dispatched action
 * @returns {promise} - thenable value.
 */
export default store => next => action => {
  //hande no action
  if (!action) return;

  const { api } = action;
  //run next action if not an API action
  if (!api) return next(action);

  //extract info from action
  const { endpoint, method = "READ" } = api;

  //merge options with default ones
  const options = { restricted: true, mute: false, ...api.options };

  //create split FSA action
  function createAction(kind, data) {
    //create action with its new type : FSA-TYPE_METHOD_(REQUEST|RESPONSE|FAILURE)
    const fsa = {
      ...action,
      ...data,
      type: `${action.type}_${method}${kind}`
    };
    //remove api to avoid recursive loops
    delete fsa.api;
    return fsa;
  }

  //trigger init action
  next(createAction(INIT));

  const state = store.getState();
  const dispatch = store.dispatch;

  //web api call
  const promise = fetch(
    computeURL(state, endpoint, false),
    httpVerbs[method],
    action.payload,
    options,
    options.restricted ? selectToken(state) : ""
  );

  return promise
    .then(response => {
      //when response comes back
      const doneAction = resultify(createAction(DONE, { payload: response }));
      //trigger done action
      next(doneAction);
      //return payload to be 'awaitable'
      return doneAction.payload;
    })
    .catch(exc => {
      //trigger fail action
      next(createAction(FAIL, { error: true, payload: exc }));

      switch (exc.number) {
        case 401: {
          //unauthorized : revoke access
          dispatch(disconnectUser(nls("core.error.unauthorizedAccess")));
          break;
        }
        case 403:
        case 404:
        case 500: {
          //navigate to error pages
          dispatch(goto(`/pages/${exc.number}`, true));
          break;
        }
        default: {
          //logs internal error if necessary
          const log = logInternalError(exc);
          if (log) dispatch(log);
        }
      }

      //continue rejection if not mute to ensure reject status can be 'catch' in user land call
      if (!options.mute) {
        return Promise.reject(exc);
      }
    });
};

function resultify(action) {
  const { payload, meta } = action;

  //case non compliant response
  if (!payload || typeof payload !== "object") return action;

  const { Code, Data, Status, Ops } = payload;
  //case classic action
  if (Code === undefined) return action;

  //throw error if status found
  if (Status) {
    throw makeInternalError(
      Status.Message,
      Status.ErrorCode,
      //warning if technical error
      Status.Code === 1 ? 2 : 3,
      Data
    );
  }

  //when generic Result<T>
  return {
    ...action,
    //replace current payload with Result<T> data
    payload: Data,
    //append resting stuff (operations) into meta descriptor
    meta: Object.assign({ Ops }, meta)
  };
}
